'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global  */

const themes = {
  fallback: 'default',
  soon: true,
  unlocked: ['default', 'dark'],
  settings: {
    default: {
      styles: {
        color: '#666',
        block_count_color: '#F30D12',
        hover_color: '#333',
        background: '#FFF',
        icon_color: '#999',
        icon_hover_color: '#666',
        warning_color: '#FC1C1C',
        control_color: '#FFF',
        control_background: '#FC1C1C',
        control_hover_background: '#cb1c1c',
        link_color: '#1855ED',
        link_hover_color: '#F30D12',
        dark_color: '#2A2A2A',
        dark_background: '#E6E6E6',
        border_color: '#ddd',
        dark_border: '#ddd',
      },
    },
    dark: {
      styles: {
        color: '#F4F4F4',
        block_count_color: '#F30D12',
        hover_color: '#F4F4F4',
        background: '#666',
        icon_color: '#F4F4F4',
        icon_hover_color: '#BBB',
        warning_color: '#FFCA00',
        control_color: '#666',
        control_background: '#fff',
        control_hover_background: '#F4F4F4',
        link_color: '#4ECDF9',
        link_hover_color: '#03BCFC',
        dark_color: '#F4F4F4',
        dark_background: '#4C4C4C',
        border_color: '#999',
        dark_border: '#555',
      },
    },
    teaser: {
      styles: {
        color: '#666',
        hover_color: '#F4F4F4',
        background: '#F4F4F4',
        icon_color: '#999',
        icon_hover_color: '#ddd',
        warning_color: '#FC1C1C',
        link_color: '#4ECDF9',
        link_hover_color: '#03BCFC',
        dark_color: '#666',
        dark_background: '#E6E6E6',
        border_color: '#ddd',
        dark_border: '#ddd',
      },
    },
  },
};

// Theme represents a single theme and is responsible for
// UI set up and theme "deployment" to the browser.
class Theme {
  constructor(name, kind, unlocked, enabled, selected) {
    this.kind = kind;
    if (typeof name === 'string') {
      this.header = 'header';
      this.body = 'text_body';
      this.name = `${name}_theme`;
      this.themeName = name;
    } else {
      this.header = name.header;
      this.body = name.body;
      this.name = name.name;
      this.themeName = name.theme;
    }
    this.unlocked = unlocked;
    this.enabled = enabled;
    this.selected = selected;
  }

  // select makes the current theme active/inactive on the given
  // parent element and saves the selection to BG settings.
  select() {
    this.selected = true;
  }

  // apply the theme to the given DOM parent element. If no parent
  // is provided, the default is to apply to the <body> element.
  // applying a theme does not select it (use select() instead).
  apply($themeParent) {
    let $parent;
    if ($themeParent) {
      $parent = $themeParent;
    } else {
      $parent = $('body');
    }
    // Set up the theme overrides for CSS properties by loading
    // from a file.
    const { styles } = themes.settings[this.themeName];
    for (const style in styles) {
      if (Object.prototype.hasOwnProperty.call(styles, style)) {
        $parent.css(`--${style.replace(/_/g, '-')}`, styles[style]);
      }
    }
    // Replace the logo for the theme
    $parent.find('img.logo').attr('src', `/themes/${this.themeName}/core-logo-full.svg`);
  }
}

// ThemeManager organizes UI style themes in the options and popup.
class ThemeManager {
  // Create a ThemeManager with the kind of themes to manager (`popup` or `options`),
  // and the currently selected theme (by name).
  constructor(kind, selected) {
    this.kind = kind;
    this.unlocked = []; // unlocked themes
    this.all = []; // all themes
    const { unlocked } = themes;
    for (let i = 0; i < unlocked.length; i++) {
      const name = unlocked[i];
      const theme = new Theme(name, this.kind, true, true, name === selected);
      this.unlocked.push(theme);
      this.all.push(theme);
      // Set up the fallback theme used as the default by the find() method.
      if (this.fallback) {
        // We have a fallback, but we use the `default` theme as the fallback if we can
        if (theme.themeName === 'default') {
          this.fallback = theme;
        }
      } else {
        // if we don't have a fallback yet, we take any valid theme as the fallback
        this.fallback = theme;
      }
    }
    if (themes.soon) {
      this.teaser = new Theme({ body: 'more_themes_coming_soon', name: 'stay_tuned', theme: 'teaser' }, this.kind, true, false, false);
      this.unlocked.push(this.teaser);
      this.all.push(this.teaser);
    }
  }

  // find locates a theme by name and returns it or the fallback/default theme if no matching
  // theme is found.
  find(name) {
    for (let i = 0; i < this.all.length; i++) {
      const theme = this.all[i];
      if (theme.themeName === name) {
        return theme;
      }
    }
    return this.fallback;
  }
}
